<?xml version="1.0" encoding="UTF-8"?>
<book version="5.0" xmlns="http://docbook.org/ns/docbook" xmlns:xi="http://www.w3.org/2001/XInclude">
<#import "utils.ftl" as utils>
<#-- Import common macros and functions -->
<#import "macros_common_general.ftl" as com>
<#-- Initialize common variables -->
<@com.initializeMainVariables/>	
<#assign rootEntity = entity.root>
<#-- Utility list to remember tarversed entities -->
<#assign traveresedEntityKeyList = [rootEntity.documentKey]>
<#-- Utility list to remember entities yet to be traversed -->
<#assign toTravereseEntityList = []>

<info>
      <title>Confidentiality Report</title>
		<cover>
		

            <#if _subject.documentType=="MIXTURE">Mixture: <@com.text _subject.MixtureName/>
            <#elseif _subject.documentType=="SUBSTANCE">Substance: <@com.text _subject.ChemicalName/>
            </#if>

                <#if _dossierHeader?has_content>
                  <#assign dossier = _dossierHeader />

                <@com.emptyLine/>

                <para>
                  <emphasis role="bold">Dossier details:</emphasis>
                    <para role="indent">
                    <itemizedlist>
                        <listitem>Dossier name: <@com.text dossier.name /></listitem>
                        <listitem>Dossier UUID: <@com.text dossier.subjectKey /></listitem>
                        <listitem>Dossier submission remarks: <@com.text dossier.remarks /></listitem>
                        <listitem>Dossier creation date and time: <@com.text dossier.creationDate /></listitem>
                        <listitem>Submission type: <@com.text dossier.submissionType /></listitem>
                        <#if ownerLegalEntity?has_content>
                          <listitem>Submitting legal entity:
                            <@com.text ownerLegalEntity.GeneralInfo.LegalEntityName/>
                          </listitem>
                        </#if>
                    </itemizedlist>
                    </para>
                </para>

                </#if>
    </cover>
 </info>
 
<chapter label="1">
		<title role="HEAD-1">Confidentiality Claims</title>
	
<#-- Go throught the found referenced entities and traverse them. NOTE: the list can grow further as entities can reference further entities. Unclear whether this works well in FreeMarker -->
<#-- FreeMarker does not support do-while loops as such do this workaround with maxLoop -->
<#assign maxLoop=5>

<#assign index=1>
<para role="small">

 <table border="1">
        <title>Confidentiality claims</title>

         <col width="3%" />
         <col width="10%" />
         <col width="15%" />
         <col width="17%" />
         <col width="10%" />
         <col width="40%" />
         <col width="5%" />

         <tbody>
            <tr>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">#</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Entity</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">IUCLID section</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Document</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Field</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Claim and Justification</emphasis></th>
              <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?><emphasis role="bold">Accept Y/N</emphasis></th>
            </tr>

            <#-- Traverse the main entity -->
            <@traverseEntity rootEntity />

              <#list 1..maxLoop as x>
                <#-- Remove documents that have been already traversed -->
                <#assign toTravereseEntityList = utils.removeDocumentWitKey(toTravereseEntityList, traveresedEntityKeyList) >
                <#if !(toTravereseEntityList?has_content)>
                  <#-- Stop looping if there is nothing to traverse -->
                  <#break>
                </#if>
                <#-- Travers each entity from the list -->
                <#list toTravereseEntityList as entityItem>
                  <#assign traveresedEntityKeyList= traveresedEntityKeyList + [entityItem.documentKey]>
                  <@traverseEntity entityItem />
                </#list>
            </#list>
            </tbody>
      </table>
</para>
<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->

<#macro traverseEntity entityDoc>
  <#if utils.isCompositeEntityDocument(entityDoc)>
    <@traverseCompositeEntity entityDoc />
  <#else>
    <@traverseSimpleEntity entityDoc />
  </#if>
</#macro>

<#macro traverseSimpleEntity entityDoc>
  <@traverseDoc entityDoc entityDoc 0 />
</#macro>

<#macro traverseCompositeEntity entityDoc>
  <#local tableOfContents=iuclid.localizeTreeFor(entityDoc.documentType, entityDoc.submissionType, entityDoc.documentKey, utils.getInheritedTemplates(entityDoc), utils.getRelatedCategories(entityDoc), utils.getRelatedMixtures(entityDoc))>
  <@traverseToc tableOfContents entityDoc 0 />
</#macro>

<#macro traverseToc sectionNode entityDocument level=0>
  <#if sectionNode?has_content>
    <#-- iterate through the section documents under this section if there are any -->
    <#if sectionNode.content?has_content>
      <#list sectionNode.content as doc>
        <#-- traverse through the document's blocks and fields aka elements -->
        <@traverseDoc doc entityDocument level+1 sectionNode />
      </#list>
    </#if>
    <#-- iterate through the child sections of the node and do a recursive call -->
    <#if sectionNode?children?has_content>
      <#list sectionNode?children as child>
        <@traverseToc child entityDocument level+1 />
      </#list>
    </#if>
  </#if>
</#macro>

<#macro traverseDoc docElementNode entityDocument level=0 sectionNode="">
  <#if docElementNode?has_content>
    <@printIfDataProtectionElementNode docElementNode entityDocument sectionNode />
    <#-- Check if the element node is a reference field (single or multiple) and remember referenced entities for later traversal in toTravereseEntityList -->
    <#assign toTravereseEntityList = utils.checkAndRememberReferencedEntity(docElementNode, toTravereseEntityList, traveresedEntityKeyList) />
    <#-- iterate through the child structure elements of this document element if there are any and do a recursive call -->
    <#if docElementNode?children?has_content>
      <#list docElementNode?children as child>
        <@traverseDoc child entityDocument level+1 sectionNode />
      </#list>
    </#if>
  </#if>
</#macro>

<#macro printIfDataProtectionElementNode docElementNode entityDocument sectionNode>
  <#if docElementNode?node_type == "data_protection">
    <@printDataProtectionField docElementNode entityDocument sectionNode />
  </#if>
</#macro>

<#macro printDataProtectionField fieldNode entityDocument sectionNode>
  <#local sectionName = "">
  <#if sectionNode?has_content>
    <#if sectionNode.number?has_content>
      <#local sectionName = sectionNode.number + " ">
    </#if>
    <#local sectionName = sectionName + sectionNode.title>
  </#if>
  <#-- The root of the fieldNode is the node that represents the Document to which the field belongs to -->
  <#local sectionDoc = fieldNode?root>
  <#local sectionDocType = sectionDoc.documentType>
  <#if !utils.isEntityDocument(sectionDoc)>
    <#local sectionDocType = sectionDocType + "." + sectionDoc.documentSubType>
  </#if>

  <#-- The field path -->
  <#local fieldPath = fieldNode._path>
  <#-- The field label -->
  <@iuclid.label for=fieldNode var="fieldLabel"/>

  <#-- The web URL pointing to the document -->
  <#assign docUrl=utils.getDocumentUrl(sectionDoc) />
  
  <#local confidentiality = utils.getPhraseCodeValue(fieldNode.confidentiality)>  
  <#local confLegislations = utils.getTextForConfidentialityLegislations(fieldNode.legislations)>
  <#assign confJustification><@com.text fieldNode.justification/></#assign>

      <#if fieldNode.confidentiality?has_content>
        <@tableOutput sectionName entityDocument sectionDoc docUrl fieldNode fieldPath confidentiality confLegislations confJustification />
        <#assign index=index+1/>
      </#if>
</#macro>

<#macro tableOutput sectionName entityDocument sectionDoc docUrl fieldNode fieldPath confidentiality confLegislations confJustification>
 <tr>
    <td>${index}</td>

     <#-- IUCLID Entity type and name -->
    <td>
        ${entityDocument.documentType?replace("_", " ")}:
        <?linebreak?>
        <@com.text entityDocument.name/>
    </td>
    
    <#-- IUCLID section name -->
    <td>
      <@com.text sectionName />        
    </td>

    <td>
       <ulink url="${docUrl}">
          <@com.text sectionDoc.name />
        </ulink>
    </td>

     <td>
         <#-- Remove last element-->
         <#local field=fieldPath?split(".")/>
         <#local field=field[0..field?size-2]/>
         <#local field=field?join(" / ")/>

         <#-- Remove doc type name-->
         <#local field = field?replace("${sectionDoc.documentType}[ / ]*", "", "r")?replace("${sectionDoc.documentSubType}[ / ]*", "", "r")/>

         <#-- Change name-->
         <#if field=="AdministrativeData" || field=="AdministrativeDataSummary" || field=="ConfidentialityInfo" || field="">
             <#local field="Full document"/>
         <#else>
             <#local field=field?replace("([A-Z]{1})", " $1", "r")?lower_case?cap_first/>
         </#if>

         <@com.text field/>

     </td>

    <td> 
      [${confidentiality}] ${confJustification}
    </td>

    <#-- empty field -->
    <td></td>

  </tr>
</#macro>
</chapter>
</book>
